

    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
    *                                                                 *
    *   Transparentizer 1.31                                          *
    *   Copyright  2004 Sebastian Pipping <webmaster@hartwork.org>   *
    *                                                                 *
    *   --> http://www.hartwork.org                                   *
    *                                                                 *
    *                                                                 *
    *   This source code is released under LGPL.                      *
    *   See LGPL.txt for details.                        2004-08-30   *
    *                                                                 *
    \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */




#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <commctrl.h>			// ListView_GetTextBkColor, ListView_SetTextBkColor
#include <shellapi.h>			// Shell_NotifyIcon

#include "resource.h"
#include "shared.h"
#include "devil_compile.h"



#define APP_TITLE				"Transparentizer"
#define APP_WNDCLASS			"TRANSPARENTIZER"

#ifndef _DEBUG
#define APP_VERSION				"1.31"
#else
#define APP_VERSION				"1.31 DEBUG"
#endif

#define WM_ICONTOUCH			( WM_USER + 1 )

#define ITEM_ABOUT				0
#define ITEM_REFRESH			1
#define ITEM_LINE				2
#define ITEM_EXIT				3

#define TIMER_ANIMATION			0
#define TIMER_SURVIVE			1

#define SURVIVE_CHECK_ICON		0
#define SURVIVE_WAIT_TRAY		1
#define SURVIVE_WAIT_DESKTOP	2
#define SURVIVE_RECREATE_ICON	3

#define LEVEL_MUTEX				0
#define LEVEL_TRANS				1
#define LEVEL_WNDCLASS			2
#define LEVEL_DLL				3
#define LEVEL_FUNCTIONS			4
#define LEVEL_HOOK				5
#define LEVEL_ICON				6



typedef bool ( * INSTALLHOOK )( HINSTANCE, DWORD, HWND, HWND );
typedef void ( * REMOVEHOOK )();

INSTALLHOOK InstallHook = NULL;
REMOVEHOOK RemoveHook = NULL;



NOTIFYICONDATA	g_NID;
HINSTANCE		g_hDLL				= NULL;
HINSTANCE		g_hInstance			= NULL;
HWND			g_hListView			= NULL;
HANDLE			g_hMutex			= NULL;
HMENU			g_hMenu				= NULL;
COLORREF		g_OriginalColor		= 0;
UINT			g_uFrame			= 0;
UINT			g_uSurviveState		= 0;
UINT			g_uWaitCounter		= 0;
bool			g_bMenuDisabled		= false;



/******************************************************************************************************
**  FindListView                                                                                     **
******************************************************************************************************/

bool FindListView( HWND * hListView )
{
	// Progman
	HWND h = FindWindow( "Progman", NULL );
	if( h == NULL )
	{
		return false;
	}

	// Child
	h = GetWindow( h, GW_CHILD );
	if( h == NULL )
	{
		return false;
	}

	// Grandchild
	h = GetWindow( h, GW_CHILD );
	if( h == NULL )
	{
		return false;
	}

	*hListView = h;
	return true;
}


/******************************************************************************************************
**  WndProc                                                                                          **
******************************************************************************************************/

LRESULT CALLBACK WndProc( HWND hwnd, UINT message, WPARAM wp, LPARAM lp )
{
	switch( message )
	{
		case WM_TIMER:
		{
			switch( wp )
			{
				case TIMER_SURVIVE:
				{
					switch( g_uSurviveState )
					{
						case SURVIVE_CHECK_ICON:
						{
							BOOL b = Shell_NotifyIcon( NIM_MODIFY, &g_NID );
							if( b )
							{
								break;				// Icon exists
							}
							else
							{
								g_uSurviveState = SURVIVE_WAIT_TRAY;
							}
						}
						case SURVIVE_WAIT_TRAY:
						{
							#ifdef _DEBUG
								OutputDebugString( "SURVIVE_WAIT_TRAY" );
							#endif

							if( g_uWaitCounter >= 10 )
							{
								#ifdef _DEBUG
									OutputDebugString( "WAIT ERR -> TRAY" );
								#endif

								// Enough waiting
								KillTimer( hwnd, TIMER_SURVIVE );
								DestroyWindow( hwnd );				// -> QUIT

								break;
							}

							HWND h = FindWindow( "Shell_TrayWnd", NULL );
							if( h )
							{
								g_uWaitCounter = 0;
								g_uSurviveState = SURVIVE_WAIT_DESKTOP;
							}
							else
							{
								g_uWaitCounter++;

								break;				// Explorer still missing
							}
						}
						case SURVIVE_WAIT_DESKTOP:
						{
							#ifdef _DEBUG
								OutputDebugString( "SURVIVE_WAIT_DESKTOP" );
							#endif

							if( g_uWaitCounter >= 10 )
							{
								#ifdef _DEBUG
									OutputDebugString( "WAIT ERR -> DESKTOP" );
								#endif

								// Enough waiting
								KillTimer( hwnd, TIMER_SURVIVE );
								DestroyWindow( hwnd );				// -> QUIT
								break;
							}

							if( FindListView( &g_hListView ) )
							{
								// Remove old hook
								RemoveHook();

								// Create new hook
								DWORD dwProgmanTID = GetWindowThreadProcessId( g_hListView, NULL );
								if( !InstallHook( g_hDLL, dwProgmanTID, g_hListView, hwnd ) )
								{
									#ifdef _DEBUG
										OutputDebugString( "HOOK ERR" );
									#endif

									KillTimer( hwnd, TIMER_SURVIVE );
									DestroyWindow( hwnd );				// -> QUIT
									break;
								}

								// Refresh
								ListView_SetTextBkColor( g_hListView, g_OriginalColor );
								ListView_SetTextBkColor( g_hListView, COLORREF_TRANSPARENT );
								InvalidateRect( g_hListView, NULL, true );


								g_uWaitCounter = 0;
								g_uSurviveState = SURVIVE_RECREATE_ICON;
							}
							else
							{
								g_uWaitCounter++;

								break;
							}
						}
						case SURVIVE_RECREATE_ICON:
						{
							#ifdef _DEBUG
								OutputDebugString( "SURVIVE_RECREATE_ICON" );
							#endif

							BOOL b = Shell_NotifyIcon( NIM_ADD, &g_NID );
							if( b )
							{
								g_uSurviveState = SURVIVE_CHECK_ICON;
							}

							break;
						}
					}

					break;
				}
				case TIMER_ANIMATION:
				{
					if( g_uFrame & 1 )
					{
						// 3, 1
						g_NID.hIcon = LoadIcon( g_hInstance, MAKEINTRESOURCE( IDI_ICON_SMALL_BUSY ) );
					}
					else
					{
						// 2, 0
						g_NID.hIcon = LoadIcon( g_hInstance, MAKEINTRESOURCE( IDI_ICON_SMALL_DEFAULT ) );
					}
					Shell_NotifyIcon( NIM_MODIFY, &g_NID );

					if( !g_uFrame )
					{
						KillTimer( hwnd, TIMER_ANIMATION );
					}
					else
					{
						g_uFrame--;
					}

					break;
				}
			}

			break;
		}
		case WM_ICONTOUCH:
		{
			switch( lp )
			{
				case WM_RBUTTONDOWN:
				{
					if( g_bMenuDisabled ) break;

					// Get mouse position
					POINT MousePos;
					GetCursorPos( &MousePos );

					// Show popup menu
					SetForegroundWindow( hwnd );
					TrackPopupMenu( g_hMenu, TPM_RIGHTALIGN | TPM_BOTTOMALIGN | TPM_LEFTBUTTON, MousePos.x, MousePos.y, 0, hwnd, NULL );
					SetForegroundWindow( hwnd );

					break;
				}
			}

			return 0;
		}
		case WM_COMMAND:
		{
			switch( LOWORD( wp ) )
			{
				case ITEM_ABOUT:
				{
					g_bMenuDisabled = true;

						char szMsg[ 1024 ];
						wsprintf( szMsg, "%s %s\nCopyright  2004 Sebastian Pipping  \n<webmaster@hartwork.org>\n\n--> http://www.hartwork.org", APP_TITLE, APP_VERSION );
						MessageBox( NULL, szMsg, "About", MB_ICONINFORMATION );

					g_bMenuDisabled = false;

					break;
				}
				case ITEM_REFRESH:
				{
					g_bMenuDisabled = true;

						ListView_SetTextBkColor( g_hListView, g_OriginalColor );
						ListView_SetTextBkColor( g_hListView, COLORREF_TRANSPARENT );
						InvalidateRect( g_hListView, NULL, true );

					g_bMenuDisabled = false;

					break;
				}
				case ITEM_EXIT:
				{
					g_bMenuDisabled = true;

						DestroyWindow( hwnd );

					g_bMenuDisabled = false;

					return 0;
				}
			}

			break;
		}
		case WM_USER:
		{
			switch( wp )
			{
				case JOB_TRANSPARENTIZE:
				{
					// Update original color
					if( g_OriginalColor != ( COLORREF )lp )
					{
						g_OriginalColor = ( COLORREF )lp;
					}

					break;
				}
				case JOB_ANIMATE:
				{
					if( !g_uFrame )
					{
						g_uFrame = 3;
						SetTimer( hwnd, TIMER_ANIMATION, 150, NULL );
					}
					else if ( g_uFrame < 3 )
					{
						g_uFrame += 2;
					}

					break;
				}
			}

			break;
		}
		case WM_CREATE:
		{
			// Create popup menu
			g_hMenu = CreatePopupMenu();
			AppendMenu( g_hMenu, MF_STRING,		ITEM_ABOUT,			"About" );
			AppendMenu( g_hMenu, MF_STRING,		ITEM_REFRESH,		"Refresh" );
			AppendMenu( g_hMenu, MF_SEPARATOR, ITEM_LINE, NULL );
			AppendMenu( g_hMenu, MF_STRING,		ITEM_EXIT,			"Exit" );

			// Activate survive timer
			SetTimer( hwnd, TIMER_SURVIVE, 1500, NULL );

			break;
		}
		case WM_DESTROY:
		{
			KillTimer( hwnd, TIMER_SURVIVE );
			PostQuitMessage( 0 );

			return 0;
		}
	}
	return DefWindowProc( hwnd, message, wp, lp );
}



/******************************************************************************************************
**  CleanUp                                                                                          **
******************************************************************************************************/

void CleanUp( UINT uLevel )
{
	switch( uLevel )
	{
		case LEVEL_ICON:
		{
			// Remove icon
			Shell_NotifyIcon( NIM_DELETE, &g_NID );
		}
		case LEVEL_HOOK:
		{
			// Remove hook
			RemoveHook();
		}
		case LEVEL_DLL:
		{
			// Unload DLL
			FreeLibrary( g_hDLL );
		}
		case LEVEL_WNDCLASS:
		{
			// Unregister window class
			UnregisterClass( APP_WNDCLASS, g_hInstance );
		}
		case LEVEL_TRANS:
		{
			// Restore original color
			if( g_OriginalColor != COLORREF_TRANSPARENT )
			{
				ListView_SetTextBkColor( g_hListView, g_OriginalColor );
				InvalidateRect( g_hListView, NULL, true );
			}
		}
		case LEVEL_MUTEX:
		{
			// Release mutex
			ReleaseMutex( g_hMutex );
		}
	}
}


/******************************************************************************************************
**  WinMain                                                                                                   **
******************************************************************************************************/

int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE, PSTR, int )
{
	g_hInstance = hInstance;


	// Deny more than one instance / Create mutex
	g_hMutex = CreateMutex( NULL, true, "MUTEX_KEEPALVIE" );
	if( ( g_hMutex == NULL ) || ( GetLastError() == ERROR_ALREADY_EXISTS ) )
	{
		return 0;
	}


	// Find LISTVIEW and transparentize
	g_hListView = NULL;
	if( !FindListView( &g_hListView ) )
	{
		CleanUp( LEVEL_MUTEX );
		return 0;
	}
	g_OriginalColor = ListView_GetTextBkColor( g_hListView );
	if( g_OriginalColor != COLORREF_TRANSPARENT )
	{
		ListView_SetTextBkColor( g_hListView, COLORREF_TRANSPARENT );
		InvalidateRect( g_hListView, NULL, true );
	}


	// Register window class
	WNDCLASSEX wcNull;
	wcNull.cbSize			= sizeof( WNDCLASSEX );
	wcNull.style			= 0;
	wcNull.lpfnWndProc		= WndProc;
	wcNull.cbClsExtra		= 0;
	wcNull.cbWndExtra		= 0;
	wcNull.hInstance		= hInstance;
	wcNull.hIcon			= NULL;
	wcNull.hCursor			= NULL;
	wcNull.hbrBackground	= NULL;
	wcNull.lpszMenuName		= NULL;
	wcNull.lpszClassName	= APP_WNDCLASS;
	wcNull.hIconSm			= NULL;
	if( !RegisterClassEx( &wcNull ) )
	{
		CleanUp( LEVEL_TRANS );
		return 0;
	}


	// Create window
	HWND hwnd = CreateWindow( APP_WNDCLASS, APP_TITLE, 0, 0, 0, 0, 0, NULL, NULL, hInstance, NULL );
	if( !hwnd )
	{
		CleanUp( LEVEL_WNDCLASS );
		return 0;
	}


	// Load DLL
	g_hDLL = LoadLibrary( "Progmanhook.dll" );
	if( g_hDLL == NULL )
	{
		MessageBox( NULL, "[Progmanhook.dll] could not be loaded.", "Error", MB_ICONSTOP );
		CleanUp( LEVEL_WNDCLASS );
		return 0;
	}


	// Get function adresses
	InstallHook = ( INSTALLHOOK )GetProcAddress( g_hDLL, "InstallHook" );
	if( InstallHook == NULL )
	{
		CleanUp( LEVEL_DLL );
		return 0;
	}
	RemoveHook = ( REMOVEHOOK )GetProcAddress( g_hDLL, "RemoveHook" );
	if( RemoveHook == NULL )
	{
		CleanUp( LEVEL_DLL );
		return 0;
	}


	// Install hook
	DWORD dwProgmanTID = GetWindowThreadProcessId( g_hListView, NULL );
	if( !InstallHook( g_hDLL, dwProgmanTID, g_hListView, hwnd ) )
	{
		CleanUp( LEVEL_DLL );
		return 0;
	}


	// Add icon
	g_NID.cbSize			= sizeof( NOTIFYICONDATA );
	g_NID.hWnd				= hwnd;
	g_NID.uID				= 0;
	g_NID.uFlags			= NIF_ICON | NIF_TIP | NIF_MESSAGE;
	g_NID.uCallbackMessage	= WM_ICONTOUCH;
	g_NID.hIcon				= LoadIcon( g_hInstance, MAKEINTRESOURCE( IDI_ICON_SMALL_DEFAULT ) ); //LoadIcon( NULL, IDI_WINLOGO );
	wsprintf( g_NID.szTip, "%s %s", APP_TITLE, APP_VERSION );

	if( !Shell_NotifyIcon( NIM_ADD, &g_NID ) )
	{
		CleanUp( LEVEL_HOOK );
		return 0;
	}


	// Main loop
	MSG msg;
	while( GetMessage( &msg, NULL, 0, 0 ) )
	{
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}


	CleanUp( LEVEL_ICON );
	return 0;
}